/* Copyright (c) 2020 Skyward Experimental Rocketry
 * Author: Alberto Nidasio
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#pragma once

#include <utils/Stats/Stats.h>

#include "HoneywellPressureSensor.h"
#include "SSCDRRN015PDAData.h"

namespace Boardcore
{

/**
 * @brief Differential pressure sensor with a ±103kPa range (±15psi)
 */
class SSCDRRN015PDA final : public HoneywellPressureSensor<SSCDRRN015PDAData>
{
public:
    SSCDRRN015PDA(std::function<ADCData()> getSensorVoltage,
                  const float supplyVoltage             = 5.0,
                  const unsigned int num_calib_samples_ = 200)
        : HoneywellPressureSensor(getSensorVoltage, supplyVoltage, 103421.3594,
                                  -103421.3594)
    {
    }
};

}  // namespace Boardcore
